﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class CharacterControl : MonoBehaviour {

    private FSMSystem fsm;
    public Stack<Vector2> path;
    public Vector2 nextWaypoint;
    public Vector2 goalPos;
    public Transform interactTarget;
    public Transform collidingWith;

    public void SetTransition(Transition t) { fsm.PerformTransition(t); }

	// Use this for initialization
	void Start () {
	
        MakeFSM();
        collidingWith = null;


	}
	
	// Update is called once per frame
	void Update () {

	
	}

    private void MakeFSM()
    {
        CharacterControl character = this.GetComponent<CharacterControl>();

        IdleState idle = new IdleState(character);
        idle.AddTransition(Transition.HasDestination, StateID.Walk);
        idle.AddTransition(Transition.ReachedItem, StateID.Interact);

        WalkState walk = new WalkState(character);
        walk.AddTransition(Transition.HasDestination, StateID.Walk);
        walk.AddTransition(Transition.ReachedGoal, StateID.Idle);
        walk.AddTransition(Transition.ReachedItem, StateID.Interact);

        InteractState interact = new InteractState(character);
        interact.AddTransition(Transition.TaskCompleted, StateID.Idle);
        interact.AddTransition(Transition.HasDestination, StateID.Walk);
        
        fsm = new FSMSystem();
        fsm.AddState(idle);
        fsm.AddState(walk);
        fsm.AddState(interact);
    }

    public void FixedUpdate()
    {
        fsm.CurrentState.Reason();
        fsm.CurrentState.Act();
    }

    public void analysePath(Transform[] waypoints, int goalIndex, Vector2 goal)
    {
        goalPos = goal;
        path = new Stack<Vector2>();
        path.Push(goal);
        int next = goalIndex;
        while (next != 0)
        {
            path.Push(waypoints[next].position);
            next = waypoints [next].GetComponent<NodeScript>().index;
        }
        
      
   //     nextState = State.Walking;
        nextWaypoint = path.Pop();
        SetTransition(Transition.HasDestination);

        
    }

    void OnTriggerEnter2D(Collider2D coll){

        if (coll.CompareTag("Item"))
        {
            collidingWith = coll.gameObject.transform;

        }
    }

    void OnTriggerExit2D(Collider2D coll){

        if(coll.gameObject.transform == collidingWith){
            collidingWith = null;
        }
    }
}
