﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class CharacterScript : MonoBehaviour
{
    public enum State
    {
        Idle,
        Walking,
        Climbing,
        Interacting
    }

    public int id;
    public State currentState;
    State nextState;
    public float speed = 3;
    public Vector2 nextWaypoint;
    public Stack<Vector2> path;
    public Transform interactItem;
    InteractScript item;
    bool hasDestination = false;

    //OnCollider Enter if item we want to interact with, then interact

    // Use this for initialization
    void Start()
    {
        currentState = State.Idle;
        nextWaypoint = transform.position;
    
    }
    
    // Update is called once per frame
    void Update()
    {

        currentState = nextState;

        switch (currentState)
        {
            case State.Idle:
                Idle();
                break;
            case State.Walking:
                Walk();
                break;
            case State.Climbing:
                Climb();
                break;
            case State.Interacting:
                Interact();
                break;
        }
    }

    void Interact()
    {

        if (hasDestination)
        {
            item.stopInteract();
            nextState = State.Walking;
        }

    }

    void Idle()
    {


    }

    void Walk()
    {
        
        Vector2 moveDir = nextWaypoint - (Vector2)transform.position;
        if (moveDir.x == 0)
        {
            if (moveDir.y <= 1 && moveDir.y >= -1)
            {
                if (path.Count == 0)
                {
                    nextState = State.Idle;
                        
                } else
                {
                    nextWaypoint = path.Pop();
                }
                    
            }else
                nextState = State.Climbing;
        } 


    }

    void Climb()
    {
       if (transform.position.y == nextWaypoint.y)
        {
            nextState = State.Walking;
        }
    }

    void FixedUpdate()
    {

       if (currentState == State.Walking)
            transform.position = Vector2.MoveTowards(transform.position, new Vector2(nextWaypoint.x, transform.position.y), Time.deltaTime * speed);
       if (currentState == State.Climbing)
            transform.position = Vector2.MoveTowards(transform.position, new Vector2(transform.position.x, nextWaypoint.y), Time.deltaTime * speed);

    }

    void OnTriggerEnter2D(Collider2D coll)
    {

        //   if(currentState == State.Idle || currentState == State.Walking){
        if (coll.gameObject.CompareTag("Item"))
        {
            Transform colliding = coll.gameObject.transform;

            if (colliding == interactItem)
            {
                item = coll.gameObject.GetComponent<InteractScript>();
                item.startInteract();
                nextState = State.Interacting;
            }
        }
        //    }

    }

    public void analysePath(Transform[] waypoints, int goalIndex, Vector2 goal)
    {

        hasDestination = true;
        path = new Stack<Vector2>();
        path.Push(goal);
        int next = goalIndex;
        while (next != 0)
        {
            path.Push(waypoints [next].position);
            next = waypoints [next].GetComponent<NodeScript>().index;
        }

        nextWaypoint = path.Pop();
        nextState = State.Walking;

    }  
}
