﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class NodeScript : MonoBehaviour {

	public int index;
	public List<Transform> connections;
	public float g_score;

    void Awake(){

        findNeighbours();

    }

	// Use this for initialization
	void Start () {

	
	}
	
	// Update is called once per frame
	void Update () {
	
	}
   /// <summary>
   /// Looks at left and right to find neigbours
   /// </summary>
    public void findNeighbours(){

        bool lineOfSight = true;
        
        RaycastHit2D[] hitsRight = Physics2D.RaycastAll(this.transform.position, Vector2.right);
        checkObjects(hitsRight);
        RaycastHit2D[] hitsLeft = Physics2D.RaycastAll(this.transform.position, -Vector2.right); 
        checkObjects(hitsLeft);

    }
    /// <summary>
    /// Checks the objects in one direction for neighbours and adds them
    /// </summary>
    /// <param name="hits">Hits.</param>
    public void checkObjects(RaycastHit2D[] hits){

        bool lineOfSight = true;
        foreach (RaycastHit2D hit in hits)
        {
            if (lineOfSight)
            {
                if (hit.collider.tag == "Obstacle")
                {
                    if (!hit.collider.GetComponent<ColliderScript>().walkable)
                        lineOfSight = false;
                } else if (hit.collider.tag == "Node")
                {
                    Transform Neighbour = hit.transform;
                    addLink(Neighbour);
                }
            }
        }

    }
    /// <summary>
    /// Adds a link from this node to a neighbour and vice versa.
    /// </summary>
    /// <param name="neighbour">Neighbour.</param>
    void addLink(Transform neighbour){

        if (!this.connections.Contains(neighbour) && neighbour != this.transform){
            this.connections.Add(neighbour);
            neighbour.GetComponent<NodeScript>().addLink(this.transform);
        }
    }
    /// <summary>
    /// Deletes the link from this node to neighbour.
    /// </summary>
    void deleteLink(Transform neighbour){           
        
        if (this.connections.Contains(neighbour) && neighbour != this.transform){
            this.connections.Remove(neighbour);
        }
    }
    /// <summary>
    /// Deletes the connection from this node to neighbour and vice versa.
    /// </summary>
    void deleteConnection(Transform neighbour){
        if (this.connections.Contains(neighbour) && neighbour != this.transform){
            neighbour.GetComponent<NodeScript>().deleteLink(this.transform);
            this.connections.Remove(neighbour);
        }
    }

    //NOTE! Can't remove all connections from this node using foreach
    //use while connections[0] !=null remove connections[0] or sth
    /// <summary>
    /// Destroys the node and the outgoing links.
    /// </summary>
    public void destroyNode(){

        foreach(Transform neighbour in connections)             
            neighbour.GetComponent<NodeScript>().deleteLink(this.transform);
        Destroy(this.gameObject);
    }

	public void set_index(int ind){

		index = ind;
	}
}
