using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class PathfindingScript : MonoBehaviour
{
    public GameObject nodePrefab;
    int arraySize = 6;
    public Vector2[] waypoints;
    public Transform[] camefrom;

    // Use this for initialization
    void Start()
    {
    
    }
    
    // Update is called once per frame
    void Update()
    {
    
    }

    public int FindPath(Vector2 start, Vector2 goal)
    {

        List<NodeScript> openset = new List<NodeScript>();
        List<NodeScript> closedset = new List<NodeScript>();
        camefrom = new Transform[arraySize];
        waypoints = new Vector2[arraySize];

        float[] g_score = new float[arraySize];
        float[] f_score = new float[arraySize];

        for (int i =0; i < arraySize; i++)
            g_score [i] = 1000;
        for (int i =0; i < arraySize; i++)
            f_score [i] = 1000;

        GameObject startObj = (GameObject)Instantiate(nodePrefab, start, Quaternion.identity) as GameObject;
        NodeScript startNode = startObj.GetComponent<NodeScript>();
        startNode.set_index(0);
        openset.Add(startNode);
        g_score [startNode.index] = 0;
        f_score [startNode.index] = g_score [startNode.index] + heuristic(start, goal);

        GameObject goalObj = (GameObject)Instantiate(nodePrefab, goal, Quaternion.identity) as GameObject;
        NodeScript goalNode = goalObj.GetComponent<NodeScript>();
        goalNode.set_index(arraySize - 1);

        NodeScript current = null;

        while (openset.Count != 0) {
            int minindex = min(f_score, openset);
            foreach (NodeScript node in openset)
                if (node.index == minindex) {
                    current = node;
                    if ((Vector2)current.transform.position == goal) {
                        startNode.destroyNode();
                        goalNode.destroyNode();
                        return arraySize - 1;
                    }
                }
            openset.Remove(current);
            closedset.Add(current);

            foreach (Transform node in current.connections) {
                NodeScript neighbour = node.GetComponent<NodeScript>();
                if (closedset.Contains(neighbour))
                    continue;

                float next_g = g_score [current.index] + Vector2.Distance(current.transform.position, neighbour.transform.position);

                if ((!openset.Contains(neighbour)) || next_g < g_score [neighbour.index]) {
                    camefrom [neighbour.index] = current.gameObject.transform;
                    waypoints [neighbour.index] = current.gameObject.transform.position;
                    g_score [neighbour.index] = next_g;
                    f_score [neighbour.index] = g_score [neighbour.index] + heuristic(neighbour.transform.position, goal);
                    if ((!openset.Contains(neighbour)))
                        openset.Add(neighbour);
                }

            }

        }
        
        startNode.destroyNode();
        goalNode.destroyNode();
        Debug.Log("no path!");
        return -1;
    }

    float heuristic(Vector2 Point, Vector2 Goal)
    {
        //Manhattan distance
        return Mathf.Abs(Point.x - Goal.x) + Mathf.Abs(Point.y - Goal.y);
    }
    /// <summary>
    /// Finds the minimum
    /// </summary>
    int min(float[] a, List<NodeScript> set)
    {

        float minval = 1000;
        int lowestIndex = -1;
        foreach (NodeScript node in set)
            if (a [node.index] < minval) {
                minval = a [node.index];
                lowestIndex = node.index;
            }

        return lowestIndex;
    }
}
