﻿using UnityEngine;
using UnityEngine.UI;
using System.Collections;

public class QuestManager : MonoBehaviour
{

	public float cooldown;
	public float currentTime;
	public Quest nextQuest;
	public TextAsset nameFile;
	public TextAsset itemFile;
	private string[] names;
	private string[] items;
	public GameObject popup;
	public GameObject text;
	public GameObject failed;
	QuestLog questLog;
	public bool inMenu;
	public int itemId;
	public int coins;
	public GameObject coinText;

	// Use this for initialization
	void Start()
	{

		currentTime = 1;
		names = nameFile.text.Split('\n');
		items = itemFile.text.Split('\n');
		inMenu = false;
		questLog = GameObject.FindGameObjectWithTag("QuestLog").GetComponent<QuestLog>();
		popup.SetActive(false);
		coins = 0;
	
	}
	
	// Update is called once per frame
	void Update()
	{

		coinText.GetComponent<Text>().text = coins.ToString()+" coins";

		if (!inMenu) {

			if (currentTime > 0f) {
				currentTime -= Time.deltaTime;
			} else {
				giveQuest();
			}
		}
	
	}

	public void enableToggles(int id)
	{
		questLog.enableToggles();
		inMenu = true;
		itemId = id;
	}

	public void selectQuest(int selection)
	{

		GameObject[] items = GameObject.FindGameObjectsWithTag("Item");
		foreach (GameObject item in items) {
			InteractScript itemScr = item.GetComponent<InteractScript>();
			if (itemScr.Id == itemId) {
				itemScr.assignQuest(selection);
				inMenu = false;
				break;
			}
		}
		questLog.disableToggles();
		inMenu = false;
		
	}
	
	public void giveQuest()
	{

		createQuest();
		showPopUp();
		currentTime = cooldown;
	}

	public void createQuest()
	{

		int randomNam = UnityEngine.Random.Range(0, names.Length - 1);
		string randomName = names [randomNam];
		int randomIt = UnityEngine.Random.Range(0, items.Length - 1);
		string randomItem = items [randomIt];
		int randomTime = UnityEngine.Random.Range(2, 6);
		int randomReward = UnityEngine.Random.Range(0, 200);
		int randomPenalty = UnityEngine.Random.Range(0, 200);
		int randomNumber = UnityEngine.Random.Range(1, 10);
		nextQuest = new Quest(randomTime, randomItem, randomName, randomNumber, randomReward, randomPenalty);

	}

	public void completeQuest(int id)
	{
		coins += questLog.quests [id].reward;
		questLog.removeQuest(id);
	}

	public void cancelQuest(int id)
	{
		coins -= questLog.quests [id].penalty;
		inMenu = true;
		failed.SetActive(true);
		failed.GetComponent<FailedPanel>().open(questLog.quests[id]);

	}

	public void closeFail(){
		failed.SetActive(false);
		inMenu = false;
	}

	public void showPopUp()
	{

		inMenu = true;
		popup.SetActive(true);
		text.SetActive(true);
		text.GetComponent<Text>().text = "" + nextQuest.giverName + " requested " +
			"" + nextQuest.noOfItems + " " + nextQuest.item + " due in " +
			nextQuest.deadline + " days! What do we do now?";
	}

	public void accepted()
	{

		inMenu = false;
		popup.SetActive(false);
		questLog.addQuest(nextQuest);
	}

	public void rejected()
	{
		inMenu = false;
		popup.SetActive(false);
	}
}
